@@ -8,16 +8,18 @@
 #
 package Dist::Zilla::Plugin::Git::GatherDir;
 {
-  $Dist::Zilla::Plugin::Git::GatherDir::VERSION = '1.120970';
+  $Dist::Zilla::Plugin::Git::GatherDir::VERSION = '1.121010';
 }
 # ABSTRACT: gather all tracked files in a Git working directory
 use Moose;
 use Moose::Autobox;
 use MooseX::Types::Path::Class qw(Dir File);
-with 'Dist::Zilla::Role::FileGatherer';
+with 'Dist::Zilla::Role::Git::Repo';
+extends 'Dist::Zilla::Plugin::GatherDir';
 
 
 use Git::Wrapper;
+use File::Find::Rule;
 use File::HomeDir;
 use File::Spec;
 use Path::Class;
@@ -25,30 +27,8 @@ use Path::Class;
 use namespace::autoclean;
 
 
-has root => (
-  is   => 'ro',
-  isa  => Dir,
-  lazy => 1,
-  coerce   => 1,
-  required => 1,
-  default  => sub { shift->zilla->root },
-);
 
-
-has prefix => (
-  is  => 'ro',
-  isa => 'Str',
-  default => '',
-);
-
-
-has include_dotfiles => (
-  is  => 'ro',
-  isa => 'Bool',
-  default => 0,
-);
-
-sub gather_files {
+override gather_files => sub {
   my ($self) = @_;
 
   my $root = "" . $self->root;
@@ -59,12 +39,22 @@ sub gather_files {
 
   my @files;
   FILE: for my $filename ($git->ls_files) {
+
+    my $file = file($filename)->relative($root);
+
     unless ($self->include_dotfiles) {
-      my $file = file($filename)->relative($root);
       next FILE if $file->basename =~ qr/^\./;
       next FILE if grep { /^\.[^.]/ } $file->dir->dir_list;
     }
 
+    my $exclude_regex = qr/\000/;
+    $exclude_regex = qr/$exclude_regex|$_/
+      for ($self->exclude_match->flatten);
+    # \b\Q$_\E\b should also handle the `eq` check
+    $exclude_regex = qr/$exclude_regex|\b\Q$_\E\b/
+      for ($self->exclude_filename->flatten);
+    next if $file =~ $exclude_regex;
+
     push @files, $self->_file_from_filename($filename);
   }
 
@@ -78,16 +68,8 @@ sub gather_files {
   }
 
   return;
-}
+};
 
-sub _file_from_filename {
-  my ($self, $filename) = @_;
-
-  return Dist::Zilla::File::OnDisk->new({
-    name => $filename,
-    mode => (stat $filename)[2] & 0755, # kill world-writeability
-  });
-}
 
 __PACKAGE__->meta->make_immutable;
 no Moose;
@@ -103,7 +85,7 @@ Dist::Zilla::Plugin::Git::GatherDir - gather all tracked files in a Git working
 
 =head1 VERSION
 
-version 1.120970
+version 1.121010
 
 =head1 DESCRIPTION
 
@@ -150,6 +132,23 @@ both for files and for directories relative to the C<root>.
 
 In almost all cases, the default value (false) is correct.
 
+=head2 follow_symlinks
+
+By default, directories that are symlinks will not be followed. Note on the
+other hand that in all followed directories, files which are symlinks are
+always gathered.
+
+=head2 exclude_filename
+
+To exclude certain files from being gathered, use the C<exclude_filename>
+option. This may be used multiple times to specify multiple files to exclude.
+
+=head2 exclude_match
+
+This is just like C<exclude_filename> but provides a regular expression
+pattern.  Files matching the pattern are not gathered.  This may be used
+multiple times to specify multiple patterns to exclude.
+
 =for Pod::Coverage gather_dir
     gather_files
 
